import { animate, AUTO_STYLE, state, style, transition, trigger } from '@angular/animations';
import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { DateTime } from 'luxon';
import { lastValueFrom } from 'rxjs';
import { structureFormStep } from '../../form/form-view/structure-form/structureFormStep.enum';
import { Owner } from '../../models/owner.model';
import { Structure } from '../../models/structure.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { WindowScrollService } from '../../shared/service/windowScroll.service';
import { AccessModality } from '../../structure-list/enum/access-modality.enum';
import { Equipment } from '../../structure-list/enum/equipment.enum';
import { Category } from '../../structure-list/models/category.model';
import { SearchService } from '../../structure-list/services/search.service';
import { FreeWorkShopLabels } from '../../structure/enums/freeWorkshop.enum';
import { FormUtils, IStructureSummary } from '../../utils/formUtils';
import { Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';

@Component({
  selector: 'app-structure-edition-summary',
  templateUrl: './structure-edition-summary.component.html',
  styleUrls: ['./structure-edition-summary.component.scss'],
  animations: [
    trigger('show', [
      state('true', style({ height: AUTO_STYLE, visibility: AUTO_STYLE, margin: '8px 0' })),
      state('false', style({ height: '0px', visibility: 'hidden', margin: '0' })),
      transition('true => false', animate('300ms ease-out')),
      transition('false => true', animate('300ms ease-out')),
    ]),
  ],
})
export class StructureEditionSummaryComponent implements OnInit {
  public structure: Structure;
  public structureForm: UntypedFormGroup;
  public members: Owner[];
  public offerHolders = [];
  public summary: IStructureSummary[] = new FormUtils().structureSummary;
  public structureFormStep = structureFormStep;
  public equipmentEnum = Equipment;
  public trainingCategories: Map<string, Category>;
  public FreeWorkShopLabels = FreeWorkShopLabels;
  public freeWorkShop: string;
  public freenessCondition: string = null;

  // Digital learning
  private categories: Category[];
  public mailHref: string;
  // set to true when last updated date of a structure is > 6 months
  public isUpdateStructure = false;
  private isGoToEdit = false;

  /** Modal canExit var */
  public updateModal = false;
  private resolve: Function;

  constructor(
    public profileService: ProfileService,
    private searchService: SearchService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private structureService: StructureService,
    private windowScrollService: WindowScrollService,
    public utils: Utils,
    public router: Router,
  ) {}

  // There is a non-blocking error in console when open edit mode that occurs only in development mode, please refer https://angular.io/errors/NG0100 for more info

  ngOnInit(): void {
    this.windowScrollService.scrolledToBottom = false;
    // eslint-disable-next-line rxjs/no-async-subscribe
    this.route.data.subscribe(async (data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        this.structureForm = new FormUtils().createStructureForm(this.structure, true);
        if (this.structure.contactMail) this.mailHref = `mailto:${this.structure.contactMail}`;

        const updatedAt = DateTime.fromISO(this.structure.updatedAt);
        const sixMonthsAgo = DateTime.local().minus({ month: 6 });
        if (updatedAt < sixMonthsAgo) this.isUpdateStructure = true;
        this.freeWorkShop = this.structure.categoriesDisplay.freeWorkShop[0]?.displayText;
        this.freenessCondition = this.structure.categories.freenessCondition as unknown as string;

        this.categories = await this.searchService.getCategories().toPromise();

        const structureWithOwners = await lastValueFrom(
          this.structureService.getStructureWithOwners(data.structure._id),
        );
        this.members = structureWithOwners.owners;

        // Get personal offers for each member of the structure
        const userId = this.profileService.getId();
        this.members.forEach((member) => {
          // Skip the user if he can't have personal offer (he may have a personal offer stored in db for this structure, but not have anymore a job with personal offer)
          if (!member.job?.hasPersonalOffer) {
            return;
          }

          // Get personal offer if the user has one in this structure
          const personalOffer = this.structure.personalOffers.find((structureOffer) =>
            member.personalOffers.some((offer) => offer._id === structureOffer._id),
          );

          // Get categories labels
          if (personalOffer) {
            // Check if it is the user's own offer
            const isMyself = userId === member._id;

            const offerHolder = {
              user: member,
              isMyself: isMyself,
              offers: this.utils.setServiceCategories(this.categories, personalOffer),
            };

            if (isMyself) {
              // If myself, add to the beginning of the array
              this.offerHolders.unshift(offerHolder);
            } else {
              // Add to the end of the array
              this.offerHolders.push(offerHolder);
            }
          }
        });

        // If the connected user has no offer for this structure, add empty offer to the beginning of the array so they can edit it
        if (!this.offerHolders.some((offerHolder) => offerHolder.user._id === userId)) {
          // Check if the connected user can have personal offer and if he's member of the structure (admin can edit the structure but is not member of the structure)
          const member = this.members.find((member) => member._id === userId);
          if (member.job?.hasPersonalOffer && member) {
            this.offerHolders.unshift({
              user: member,
              isMyself: true,
              offers: {
                categories: { advancedSkills: [], baseSkills: [], onlineProcedures: [] },
                categoriesDisplay: { advancedSkills: [], baseSkills: [], onlineProcedures: [] },
              },
            });
          }
        }

        // Add the structure offer to the beginning of the array (in first position, before the connected user)
        this.offerHolders.unshift({
          structure: this.structure,
          offers: this.structure,
        });

        this.initCategories();
      }
    });
  }

  async initCategories(): Promise<void> {
    const skillsMap = new Map<string, Category>();
    this.categories
      .filter((category) => category.id === CategoryEnum.baseSkills || category.id === CategoryEnum.advancedSkills)
      .forEach((category) => {
        skillsMap.set(category.id, category);
      });

    this.trainingCategories = skillsMap;
  }

  public goBack(): void {
    history.back();
  }

  public goToEdit(step: structureFormStep, offer: any = null): void {
    this.isGoToEdit = true;
    if (offer?.user) {
      if (offer.offers._id) {
        this.router.navigate([`/profil/edition-offre-personnelle/${offer.offers._id}`], {
          state: {
            structure: this.structure,
            tab: step === structureFormStep.structureTrainingType ? 'digitalSkills' : '',
          },
        });
      } else {
        this.router.navigate(['/formulaire/offre-personnelle'], {
          state: {
            structure: this.structure,
            page: step === structureFormStep.structureTrainingType ? 'personalOfferTrainingType' : '',
          },
        });
      }
    } else {
      this.router.navigate(['/formulaire/structure', this.structure.permalink, structureFormStep[step]]);
    }
  }

  public canExit(): Promise<boolean> {
    if (this.isUpdateStructure && !this.isGoToEdit) {
      return new Promise((resolve) => this.showUpdateModal(resolve));
    }
  }
  private showUpdateModal(resolve: Function): void {
    this.updateModal = true;
    this.resolve = resolve;
  }

  public hasRedirectionAccepted(hasAccept: boolean): void {
    // !hasAccept to invert usual modal confirmation logic (here stay on page if user validates modal, and quit if user cancels)
    this.resolve(!hasAccept);
    this.updateModal = false;
  }

  public isFormValid(): boolean {
    for (const field in this.structure) {
      if (!['_id', 'numero', 'createdAt', 'updatedAt'].includes(field)) {
        if (this.structureForm.get(field)?.valid === false) return false;
      }
    }
    // For structure not updated for 6 months, validation button is disabled until user scrolls to bottom
    if (this.isUpdateStructure && !this.windowScrollService.scrolledToBottom) {
      return false;
    }
    return true;
  }

  public onLastButtonKeydown(event: KeyboardEvent): void {
    switch (event.key) {
      case 'ArrowUp':
      case 'ArrowDown':
      case 'Tab':
        // For keyboard navigation, at the last button of the page, we consider the user reached the bottom of the page
        this.windowScrollService.scrolledToBottom = true;
        break;
    }
  }

  public updateStructureUpdateDate(): void {
    this.structureService.editStructure({}, this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('La structure a bien été mise à jour.');
      this.isUpdateStructure = false;
    });
  }

  public formatAddress(): string {
    let address = '';
    if (this.structure.address.numero) address += `${this.structure.address.numero} `;
    if (this.structure.address.street) address += `${this.structure.address.street}, `;
    if (this.structure.address.postcode) address += `${this.structure.address.postcode} `;
    if (this.structure.address.commune) address += `${this.structure.address.commune}`;
    return address;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    return this.utils.getAccessLabel(accessModality);
  }

  public containsHours(): boolean {
    for (const day in this.structure.hours) {
      if (this.structure.hours[day].time.length) return true;
    }
    return false;
  }

  public isFieldValid(field: string, theme?: string): boolean {
    if (theme) {
      return this.structureForm.get(theme).get([field]).valid;
    }
    return this.structureForm.get([field]).valid;
  }

  public requiredPhoneOrMailError(): boolean {
    return this.structureForm.hasError('requirePhoneOrMail');
  }

  public hasSocialNetworks(): boolean {
    return this.utils.hasSocialNetwork(this.structure);
  }

  public hasWebsite(): boolean {
    return this.utils.hasWebsite(this.structure);
  }

  public containsDigitalHelp(offer): boolean {
    return offer.categoriesDisplay.onlineProcedures?.length > 0;
  }

  public isBaseSkills(offer): boolean {
    return offer.categories.baseSkills?.length > 0;
  }
  public isAdvancedSkills(offer): boolean {
    return offer.categories.advancedSkills?.length > 0;
  }

  public containsDigitalLearning(offer): boolean {
    return this.isBaseSkills(offer) || this.isAdvancedSkills(offer);
  }

  /**
   * Check if the structure or one of its members offers digital learning
   */
  public containsDigitalLearningInStructureOrPersonalOffers(): boolean {
    const hasDigitalLearningInOffers = this.offerHolders.some((offerHolder) =>
      this.containsDigitalLearning(offerHolder.offers),
    );

    return hasDigitalLearningInOffers || this.containsDigitalLearning(this.structure);
  }

  public hasWifi(selfServiceMaterial: string[]): boolean {
    return this.utils.hasWifi(selfServiceMaterial);
  }

  public hasEquipments(structure: Structure): boolean {
    return this.utils.hasEquipments(structure.nbComputers, structure.nbPrinters, structure.nbScanners);
  }
  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    return this.utils.getEquipmentsLabelAndValue(equipment, number);
  }

  public hasFreenessInfo(): boolean {
    return this.isFieldValid('freeWorkShop', 'categories');
  }
  public hasFreenessCondition(): boolean {
    const freenessCondition = this.structureForm.get('categories').get(['freenessCondition']).value;
    return !(this.utils.isNullOrEmpty(freenessCondition) || freenessCondition.length === 0);
  }

  public goToManageMembers(): void {
    this.isGoToEdit = true;
    this.router.navigate([`/profil/gestion-membres/${this.structure.permalink}`]);
  }

  public getStructureTypeIcon(): string {
    return this.structure?.getTypeStructureIcon();
  }
}
